# Getting Started {#getting_started}
This guide will show you the basic usage of the geometry objects.

&rarr; Only the most important operations are shown here.
You can refer to the *unit tests* within the test folder located in the source code for more examples.
The library is completely *test driven* so you can find most functionality in the tests and see how they are supposed to be used.

&rarr; The following examples assume you execute them in a sequential manner. That means that the objects which appear in the example, which are not directly instantiated, are assumed to already exists and are created in the previous code examples.

---

## Install CGAL
The **CGAL** library is an external dependency we do not provide within the copy of **UNICADO**. So you need to make sure the library is installed and can be found by *CMake*. You can refer to the [&rarr; Getting Started Guide](https://doc.cgal.org/latest/Manual/general_intro.html) of **CGAL** to find instructions for your machine.

If you use the **MSYS2** environment as the **UNICADO** build setup requires, you can install **CGAL** with this command in your terminal:
```sh
pacman -S mingw-w64-ucrt-x86_64-cgal
```

---

## Create Sections

```cpp
/* Includes */
#include <aircraftGeometry2/geometry/section.h>

/* Create the shape */
geom2::Polygon_2 shape;
shape.push_back(geom2::Point_2(0, 0));
shape.push_back(geom2::Point_2(1, 0));
shape.push_back(geom2::Point_2(1, 1));
shape.push_back(geom2::Point_2(0, 1));

/* Create the section */
geom2::PolygonSection section(shape);
```

---

## Create Surfaces
```cpp
/* Includes */
#include <aircraftGeometry2/geometry/surface.h>
#include <aircraftGeometry2/geometry/factory.h>

/* Setup the surface builder */
geom2::SectionBuilder<geom2::PolygonSection> builder;

/* Insert section using different methods */
geom2::Vector_3 offset_z{0.0, 0.0, 0.5};
geom2::Vector_3 offset_y{0.0, 0.5, 0.5};
builder.arrange(shape, offset_z, 2);
builder.insert_back(shape, offset_y);

/* Create the surface */
geom2::MultisectionSurface<geom2::PolygonSection> surface;
surface.sections = builder.get_result();
```

---

## Use Surfaces
### Measure
```cpp
/* Includes */
#include <aircraftGeometry2/processing/measure.h>

/* Measure the surface area of a surface */
double area = geom2::measure::area(surface);
```

### Transform
```cpp
/* Includes */
#include <aircraftGeometry2/processing/transform.h>

/* Create a triangulated surface mesh */
geom2::Mesh mesh = geom2::transform::to_mesh(surface.sections);
```

---

## Import Surfaces
```cpp
/* Includes */
#include <aircraftGeometry2/hull_surface.h>

/* Create the nacelle surface as defined in the Aircraft XML file.
 * => Assuming the XML file is loaded and exists as a node object.
 * => And assuming you have the path `data_dir` where the dat files are stored.
 */
geom2::HullFactory factory{AcXml, data_dir};
geom2::MultisectionSurface<geom2::PolygonSection> surface = factory.create("Nacelle@ID");
```

---