# Using the Geometry Classes {#tutorial_geometry}
The following tutorial provides you with the basic usage to create your own shapes.
For each instruction the C++ and Python code is available.
The resulting images of each step are not part of the library, though.
Those images are rendered with a 3D animation software after the geometry is exported as a `*.ply` file after each step.
The tutorial will show as a last step, how this is done.

!!! note 
    The Python examples assume that you imported the module with:
```python
import pyaircraftGeometry2 as geom2
```

---

## Hull Surface
This tutorial shows you how you can define a simple hull surface.
The hull surface is basically a tube where you can define the shape of the cross sections.

### Step 1 - Loading DAT files
The simplest way to start with a shape for the sections is to load the 2D data from a `*.dat` file.
An example file which defines the shape of a circle is provided as part of the unit tests.
The file can be found here: `aircraftGeometry2/test/stubs/circle-tab.dat`.

- Load the geometry:

<div class="tabbed">
 
- <span class="tab-title">C++</span> 
```cpp
#include <aircraftGeometry2/io/dat.h>
geom2::Polygon_2 shape = geom2::io::read_dat_file("aircraftGeometry2/test/stubs/dat-files/circle-tab.dat");
```
- <span class="tab-title">Python</span>
```python
shape = geom2.io.read_dat_file("aircraftGeometry2/test/stubs/dat-files/circle-tab.dat")
```

</div>

!!! note 
    Make sure the path to the file is correct according to your current working directory!

This will give you the following 2D polygon:

![Tutorial Step 1](Tutorial/step001.png){html: width=600}

The 2D polygon is **always** oriented in the `XY`-Plane.
The 2D geometry gets oriented in the 3D space when you use it as the section of a surface.

### Step 2 - Create Section
The imported shape just contains the coordinates of the vertices of the polygon.
You have to create a `geom2::PolygonSection` from this shape to further create geometry.

- Create a section:

<div class="tabbed">

- <span class="tab-title">C++</span>
```cpp
#include <aircraftGeometry2/geometry/section.h>
geom2::PolygonSection section(shape);
```
- <span class="tab-title">Python</span>
```python
section = geom2.PolygonSection(shape)
```

</div>

This does not change the shape of the polygon, but it gives the shape an orientation in the 3D space.
The geom2::PolygonSection class has also methods to manipulate the shape of the polygon.

### Step 3 - Create Surface
After converting the imported polygon to a section, you can use this section to build a surface.
This surface is basically a container for different PolygonSections, see geom2::MultisectionSurface.

This library uses the *builder design pattern* to build a surface.
This builder has some convenient methods to quickly create a surface.
The key concept is, that you first create the builder, perform the build steps and then you can retrieve the result from the builder to further use the surface.

- Create a surface with two sections:

<div class="tabbed">

- <span class="tab-title">C++</span>
```cpp
#include <aircraftGeometry2/geometry/factory.h>
/* Create builder */
geom2::SectionBuilder<geom2::PolygonSection> builder;

/* Instruct the builder to insert sections */
builder.insert_back(section, geom2::Vector_3(0,0,0)); // Section 0
builder.insert_back(section, geom2::Vector_3(0,0,1)); // Section 1 with offset in Z direction

/* Create the surface */
geom2::MultisectionSurface<geom2::PolygonSection> surface;
surface.sections = builder.get_result();
```
- <span class="tab-title">Python</span>
```python
# Create the builder
builder = geom2.PolygonBuilder()

# Instruct the builder to insert sections
builder.insert_back(section, geom2.Vector_3(0,0,0)) # Section 0
builder.insert_back(section, geom2.Vector_3(0,0,1)) # Section 1 with offset on Z direction

# Create the surface
surface = geom2.PolygonSurface()
surface.sections = builder.get_result()
```

</div>

@remark There is also the geom2::SectionBuilder::arrange function which can be used to insert multiple equidistant sections of the same shape.

A few things to note about the builder:
1. The section gets copy-constructed within the builder. So you can use the same section multiple times and even change some parameters in between without affecting the already inserted sections.
2. The result of the builder gets **moved**! This means, after you retrieve the result, the builder is "empty" and does not contain a valid surface anymore.
3. Instead of providing a geom2::PolygonSection as the shape input, you can directly use a geom2::Polygon_2. The builder converts the polygon to a section automatically in this case.

The resulting surface has two circular sections and looks like a cylinder.
The length of the cylinder is 1 as the defined by the offset of the second section.

![Tutorial Step 3](Tutorial/step003.png){html: width=600}

### Step 4 - Modify Section
The builder is only intended to create the basic structure of the multi-section surface with the necessary section count.
It does not provide all the flexibility which is needed to form any surface.
You can access and modify each section of the surface instead.

- Set the width of the second section:

<div class="tabbed">

- <span class="tab-title">C++</span>
```cpp
surface.sections.back().set_width(0.5);
```
- <span class="tab-title">Python</span>
```python
surface.sections[-1].set_width(0.5)
```

</div>

The top section looks like this now:

![Tutorial Step 4](Tutorial/step004.png){html: width=600}

The **width** refers to the dimension in the local `X` direction of the section.
Whereas the **height** refers to the `Y` direction.
You can set the width and height separately for geom2::PolygonSection.
The **scale** methods applies an **uniform** scaling with the origin point of the section as the scaling center.

### Step 5 - Move Section
You also change the location of the section within the surface afterwards, but **not the order of the sections!**
The location of the section within the local coordinate system of the section is defined by the origin point of the section.

- Change the location of a section:

<div class="tabbed">

- <span class="tab-title">C++</span>
```cpp
surface.sections[0].origin = geom2::Point_3(-0.5,0,0);
```
- <span class="tab-title">Python</span>
```python
surface.sections[0].origin = geom2.Point_3(-0.5,0,0);
```

</div>

!!! note 
    **CGAL** does not supply a mechanism to change single components of points or vectors. You always have to assign a complete new set of coordinates. However, you can read the value of single coordinate components.

As you can see, the bottom section moved in negative X direction:

![Tutorial Step 5](Tutorial/step005.png){html: width=600}

### Step 6 - Orient Surface
So far, we only operated in the local coordinate system of the multi-section surface.
The surface can be aligned in the global 3D space using its **origin** and, most importantly, its **normal** direction.
The main extrusion axis within the **local coordinate** system of the surface is always the `Z` direction, as you can see in the previous steps.
You can specify the orientation of the *local* `Z` axis within the *global* 3D space by defining the direction of the **normal**.
The **normal** defacto represents the orientation of the `Z` axis.

@attention It is not enough to just specify the `Z` direction to fully define the orientation of geometry within the 3D space. This only defines 2 of the 3 necessary Euler angles, the third is technically undefined! See @ref euler_angles for more details. The library assumes the third angle to be 0°, which leads most of the times to the expected result.

- Orient the surface along global X direction:

<div class="tabbed">

- <span class="tab-title">C++</span>
```cpp
surface.normal = geom2::Direction_3(1,0,0);
```
- <span class="tab-title">Python</span>
```python
surface.normal = geom2.Direction_3(1,0,0);
```

</div>

In this example the result is this:

![Tutorial Step 6](Tutorial/step006.png){html: width=600}

The highlighted coordinate system in the figure is now the **global** coordinate space and not the local any more.
The local coordinate system of the surface remains unchanged.
You can also see that the orientation introduced a rotation around the local `Z` axis in this case, again see @ref euler_angles why this is.
For the expected result you need to set the parameter `rotate_z` of the surface.

!!! note 
    This behavior is admittedly not the most intuitive result. In further release this might get fixed. In a practical point of view, you are most often interested in properties within the local coordinate system of the surface rather then global properties in between multiple surfaces.

### Step 7 - Move Surface
As for moving the location of a section, the same principle applies when moving the surface within the global 3D space.
The **origin** of the surface defines the global position.
The **origin** is located at the origin of the first section.

- Move surface

<div class="tabbed">

- <span class="tab-title">C++</span>
```cpp
surface.origin = geom2::Point_3(0,-0.25,0);
```
- <span class="tab-title">Python</span>
```python
surface.normal = geom2.Point_3(0,-0.25,0);
```

</div>

This moves the surface *0.25* in negative `Y` direction:

![Tutorial Step 7](Tutorial/step007.png){html: width=600}

### Step 8 - Measure Surface
After creating the geometry, you can use different measurement tools to extract dimensions.
The measurement tools always reference the surface in its local coordinate system.
So the results do not depend on the global orientation of the surface.

- Measure the width of the surface at local `Z` position 0.5:

<div class="tabbed">

- <span class="tab-title">C++</span>
```cpp
#include <aircraftGeometry2/processing/measure.h>
double width = geom2::measure::width(surface, 0.5);
```
- <span class="tab-title">Python</span>
```python
width = geom2.measure.width(surface, 0.5)
```

</div>

This should result in `width = 0.75`.

!!! note 
    There are measurement functions which use additional properties of the geom2::MultisectionSurface as their input, because their result depends on more information than contained in the *sections* vector. See the documentation of each measurement for more information.

### Step 9 - Export PLY (optional)
As an optional step, you can export your surface as a triangulated surface mesh in the **PLY** format.

- Export mesh:

<div class="tabbed">

- <span class="tab-title">C++</span>
```cpp
#include <CGAL/Surface_mesh/IO/PLY.h>
geom2::Mesh mesh = geom2::transform::to_mesh(surface);
CGAL::IO::write_PLY("./mesh.ply", mesh);
```
- <span class="tab-title">Python</span>
```python
geom2.io.export_ply(surface, "./mesh.ply")
```

</div>

!!! note 
    This function is rather a debugging tool than a fully tested tool for production use!

---

## Airfoil Surface {#tutorial_airfoil_surface}
An airfoil surface works similar to the hull surface.
Instead of using polygon sections, the airfoil surface expects geom2::AirfoilSection as inputs.
Those sections follow the same principle as the polygon sections, but offer different methods which are more suitable/intuitive for airfoil shapes.

- You can load an airfoil files with the corresponding function:

<div class="tabbed">
 
- <span class="tab-title">C++</span> 
```cpp
#include <aircraftGeometry2/io/dat.h>
geom2::Polygon_2 shape = geom2::io::read_airfoil("aircraftGeometry2/test/stubs/n0012-tab.dat");
```
- <span class="tab-title">Python</span>
```python
shape = geom2.io.read_airfoil("aircraftGeometry2/test/stubs/n0012-tab.dat")
```

</div>

This reads the dat file **and** additionally sorts the points that the resulting polygon does not intersect itself, because airfoil coordinate files specify first the upper coordinates and then the lower coordinates which would result in a discontinuity in between the upper and lower part.

- Create an initial airfoil surface:

<div class="tabbed">

- <span class="tab-title">C++</span>
```cpp
#include <aircraftGeometry2/geometry/factory.h>
/* Create builder */
geom2::SectionBuilder<geom2::AirfoilSection> builder;

/* Instruct the builder to arrange 3 sections */
builder.arrange(section, geom2::Vector_3(0,0,-1), 3);

/* Create the surface */
geom2::MultisectionSurface<geom2::AirfoilSection> surface;
surface.sections = builder.get_result();
```
- <span class="tab-title">Python</span>
```python
# Create the builder
builder = geom2.AirfoilBuilder()

# Instruct the builder to arrange 3 sections
builder.arrange(section, geom2.Vector_3(0,0,-1), 3)

# Create the surface
surface = geom2.AirfoilSurface()
surface.sections = builder.get_result()
```

</div>

This gives a planar rectangular wing shape:

![Tutorial Airfoil Surface](Tutorial/airfoil_surface.png){html: width=600}

@attention You should always use the **negative** `Z` direction as the main span direction of an airfoil surface. Only then do the angles like twist or dihedral rotate in the direction you would expect.
