# Systems Design Software Architecture 
If you are interested in how **systems_design** performs the system sizing and power requirment calculation, this page will give you an overview. 
## Module Structure
Currently **systems_design** only has one strategy - the `STANDARD` strategy implemented in `standardSystemsDesign.cpp`. In **systems_design** each system is represented by an instance of the class `aircraftSystem`. Its properties include the point mass and CoG of the system, the required power, and the heat load emitted by the system. Each aircraftSystem has functions for calculating these properties. The power of a system is not constant throughout the mission but calculated for each mission step. These values are stored in objects of the class `powerProfile`. This class stores the design power, used for system sizing, and the mission power. The mission power is further divided into the base load during this mission step and potential peak loads, e.g. resulting from the retraction or extension of the landing gear. Heat loads occurring due to energy losses in the systems are also calculated for each mission step and stored in a powerProfile.

If a new aircraftSystem is implemented, its header file has to be included in `standardSystemsDesign.cpp` and an else-if-statement calling its constructor according to the system name has to be added to `standardSystemsDesign::initializeSystems()`.

## Calculation
#### Initialize
The system design module starts with `standardSystemsDesign::initialize()` by initializing all aircraft systems using the user-provided data from the configuration file. The config file is read when the `data_` object is created. Additionally, required aircraft parameters from the aircraft exchange file are read. This includes geometry data (fuselage, nacelles, wing, empennage), propulsion data, mass and performance and accomodation data. This data are required for the power and mass calculations of specific systems. However, all data are read centrally by the `systemsIOData` class during initialization. Lastly checks for the user input and the geometry from the acxml are performed.

**systems_design** considers the temperature offset to the International Standard Atmosphere (ISA) defined in the _acXML_. The resulting temperature changes in the atmospheric conditions are applied to the mission steps and affect the power demand and thus the mass properties of systems depending on atmospheric conditions ([environmental control system](\ref ATA21-ECS), [flight control system](\ref ATA27-flight-control), [ice and rain protection system](\ref ATA30-ice-conventional)).

#### Run
\anchor modes
The strategy standard systems design contains three modes, which can be selected in the configuration file. The sizing mode calculates the required system power and based on that the system masses. The study and requirment mode can be used to calculate the required system power for different missions (either the study or requirments mission) and do not resize the systems (i.e., no mass calculation is performed).

**Sizing Mode**

First, the module calculates the performance profile of each system (with the function `standardSystemsDesign::calculatePerformanceProfile()`). This involves calling the respective power calculation function for each system in a specific order: consumer systems, the environmental control system (ECS), conducting systems, and source systems. The calculation order is crucial because the power requirements of consumer systems are necessary to calculate the conducting and source power. At first, the power profiles of all consumer systems are calculated (`standardSystemsDesign::getSinkEnergyConsumption()`) except for the ECS. It is treated separately because its power requirement depends on the heat loads generated by other systems, including conducting systems. Therefore, it is calculated later in the process.

After each consumer system power calculation, the power is accumulated in a global power profile that stores the combined power requirement and heat loads of all consumer systems. Conductor systems can provide power to other conducting systems, e.g. through the conversion of electrical power to hydraulic power. The power used for these conversions is calculated next (with `transferElec2HydPower` (electric driven hydraulic pumps) and `transferHyd2ElecPower` (only for failure cases)) and stored in the global sink power profile.

Next, an iteration loop is required because the ECS power requirement depends on the heat load of the sinks and conductors and the conducting power depends on the power requirement of the ECS. At the beginning of each loop the power profiles `data_->data.ECSConsumption` and `data_->data.conductorConsumption` are cleared from the values of the previous iteration loop and the global power profiles of the conductors are initialized (`data_->Systems.energyConductor`). In each iteration loop, the ECS power is calculated (`standardSystemsDesign::getECSEnergyConsumption()`) based on the heat loads of the consumer systems stored in the global sink power profile. The power requirements for conducting systems are then calculated or updated (`standardSystemsDesign::getConductorEnergyConsumption()`) to reflect the new ECS consumption, including any power transfers, such as from hydraulic to electrical power. The iteration continues until the average bleed load of the ECS converges. The convergence criterion is set to a difference of less than 10<sup>-4</sup> between iterations.

After achieving convergence, the module calculates the power the sources (`standardSystemsDesign::getSourceEnergyConsumption()`) must provide based on the power profiles of the consumer and conductor systems and the ECS. The ECS is now moved to the vector containing the energy sinks (`data_->Systems.energySink`) and that containing all systems (`data_->Systems.allSystems`), since it can now be treated as an ordinary sink system. 

Finally, with the power profiles established, the module calculates the mass of each system (`standardSystemsDesign::getSystemsMass()`) based on the design power requirements and specific characteristics of each system by calling the mass calculation method of each system. For virtual systems no method is called, rather the user definded values are copied from the configuration file. After the individual system masses are calculated, the function `weightsAndCGs::getWeights()` calculates the masses and CGs of system groups and the mass of some operator items. In UNICADO only the residual oil and fuel as well as water and toilet chemicals are considered part of the systems. Other operator items are calculated in fuselage design and mission analysis.

**Note**: The currently implemented methods for the operator items (Torenbeek) do not calculate the residual oil mass!

**Study Mode**

This mode is used to calculate the power required by the systems during the study mission. The methodology is the same as for the sizing mode up until the calculation of system masses, which are skipped. However, the study mission file (`study_mission.xml`) is used instead of the design mission file (`design_mission.xml`). This means offtakes are written to the study mission file but no changes are made to the _acXML_ since the systems are not resized.

**Requirements Mode**

This mode is used to calculate the power required by the systems during the requirements mission. The methodology is the same as for the sizing mode up until the calculation of system masses, which are skipped. However, the requirement mission file (`requirement_mission.xml`) is used instead of the design mission file (`design_mission.xml`). This means offtakes are written to the requirements mission file but no changes are made to the _acXML_ since the systems are not resized.

### Output
#### Update
The update section includes all updates to the aircraft exchange file and the mission file. If the sizing mode was run, the system masses in the aircraft exchange file are updated. Since the systems are considered as point masses in UNICADO the moments of inertia are set to zero. The function `systemsIOData::updateMassProperties()` updates the total mass of all systems and the operator items mass. The masses of the individual systems are updated by their classes, which all contain a function `updateXML()`. This way only the systems included in the architecture are written to the aircraft exchange file. 

The function `systemsIOData::updateMissionXML()` updates all values calculated with the mission data (i.e., power requirements). This are mostly values in the `mission.xml` but can also include values in the aircraft exchange file.

#### Report
This section is used to create the plots and html and tex report of the module.
Plots are generated using matplot++. Additionally, csv files with the plot data are written.
