# Calculation method
The task of the _tank_design_ module differs slightly depending on the energy carrier:

- [Kerosene](#kerosene-tanks) - Determine the maximum fuel capacity of the aircraft using its geometry.
- [Liquid hydrogen](#liquid-hydrogen-tanks) - Size tanks to ensure that the required amount of fuel is available.

## Kerosene tanks {#kerosene-tanks}
For kerosene-powered aircraft, the fuel is stored in the wing. Additional tanks can be installed in the fuselage 
(additional center tank) or in the empennage (trim tank).

### General methodology
<!-- The tank design process starts with determining the required energy amount since the energy amount is necessary to 
check whether the tank capacity is big enough to store the mission energy. If this information is not yet available
 in the _acXML_ in the first iteration loop, an initial estimate is made:
$
    E_{\text{mission}} = n_{\text{PAX}} \cdot R \cdot fc_{\text{approx}}
$
In which
- $n_{\text{PAX}}$ - number of passengers
- $R$ - range in km
- $fc_{\text{approx}}$ - approx. fuel consumption per passenger per 100 km (ca. 3.5 L/100 km) -->

The tank design starts with the wing tanks. Initially, the available volume of the complete wing is calculated which is then followed by the dimensioning of the vent tank. Afterwards, the wing is divided into several tanks based on its geometry. The gross volume of these tanks is calculated using the [obelisk method](#obelisk-method). This is followed by accounting for volume losses due to structural components and thermal expansion, resulting in the net [usable tank volume](#net-tank-volume).
The [energy stored](#calculate-energy) in the tanks is then calculated using the volumetric energy density of kerosene.
Afterwards, the remaining tanks are calculated, based on the specified tank configuration in the _acXML_.

Note, that after every tank calculation, there is an assessment whether or not the required mission energy can be 
stored in the available tanks. If this is the case, the flag `energy_capacity_required_for_mission` is set to `False` 
for all remaining tanks. This means that all tanks defined in the _acXML_ are designed, whether required to store the 
mission energy or not!

### Wing tanks
The tank capacity of aerodynamic surfaces like the wing or the empennage can be determined by a simple calculation of the volume.

At first, the overall wing tank volume is determined to get the vent tank volume required by the Certification Specification:

> CS 25.969 Fuel tank expansion space: Each fuel tank must have an expansion space of not less than 2% of the tank capacity. It must be impossible to fill the expansion space inadvertently with the aeroplane in the normal ground attitude.

Knowing the necessary vent tank volume, the remaining volume of the wing tanks can be calculated. The volume of the vent as well as the volume of any other wing tank can be determined using the obelisk method.

#### Obelisk method {#obelisk-method}
The obelisk method simplifies the wing by dividing it into several volumes. Depending on the geometry, this results in three (single trapezoidal wing) or five (double trapezoidal/kinked wing) individual wing tanks, each of which has the shape of an obelisk. In the case of a double trapezoidal geometry, it is assumed that the wing can be divided into an inner tank (wing root to kink) and an outer tank (kink to vent tank position) per side as well as a wing center tank.

![](figures/01_tank_locations.png)

The obelisk volume can be determined using two different approaches that are described in the following.
The user can select the desired method via the following node in the `program_settings` section of the _confXML_:
`configuration[@ID="tube_and_wing"]/specific/kerosene_tank_design_parameter/obelisk_calculation_method`.

!!! note 
    The obelisk method is explained using the wing as an example. The volume of a [trim tank](#trim-tank) located in the horizontal stabilizer is determined analogously.

##### Obelisk volume according to Torenbeek<sup>[1]</sup>
![](figures/02_obelisk.png)

The volume can be calculated using the following equation:

$$
    V_{\text{obelisk}} = \frac{l}{3} \cdot \left( S_1 + S_2 + \frac{h_1 \cdot w_2 + h_2 \cdot w_1}{2}\right)
$$

In which

- $l$ - length
- $S_1$, $S_2$ - end face areas
- $h_1$, $w_1$  - height and width of end face $S_1$
- $h_2$, $w_2$  - height and width of end face $S_2$

The parallel end faces $S_1$ and $S_2$ are derived from the position of the spars of the wing box, that are 
known from the wing design.
<!-- Based on the formula for determining the area of a rectangle:
$
    S_{i} = h_i \cdot w_i
$
the size of the parallel end surfaces $S_1$ and $S_2$ is obtained by extending it with the chord length 
$l_{chord}$:
$
    S_{i} = \frac{h_i}{l_{chord}} \cdot l_{chord} \cdot w_i
$
By multiplying with the maximum thickness $h_{max}$ and substituting the width $w_i = p_{fs}-p_{rs}$, we get:
$
    S_{i} = \frac{h_{max}}{l_{chord}} \cdot \frac{h_{i}}{h_{max}} \cdot l_{chord} \cdot (p_{rs} - p_{fs})
$

Where
- $\frac{h_{max}}{l_{chord}}$ - maximum thickness ratio of used profile
- $\frac{a_{max}}{d_{max}}$ - ...
- $p_{rs}$ - position rear spar
- $p_{fs}$ - position front spar

![](figures/03_wing_box.png) -->

##### Obelisk volume according to Simpson
The Simpson's rule is a method of numerical integration that is often used to calculate the volume of bodies whose 
cross-sections are known at different positions. In the case of an obelisk - i.e. a body with square or rectangular 
cross-sections that vary along the height - the volume is integrated as the sum of the cross-sectional areas $S(x)$
along the length $l$:

$$
    V_{\text{obelisk}} = \int_0^l S(x) \, dx
$$

If the cross-sectional areas $S(x)$ at $i+1$ uniformly distributed points are known (which is the case for the 
tank design), Simpson's rule can be applied.

Two end surfaces are known for each tank, so that a third middle surface $S_{12}$ can be determined by linear 
interpolation (see following figure). Each tank is thus divided into two sections.

![](figures/02_obelisk_simpson.png)

The tank volume can therefore be determined using a simplified Simpson's rule:

$$
    V_{\text{obelisk}} = \frac{l}{6.0} \cdot (S_{1} + 4.0 \cdot S_{12} + S_{2})
$$

#### Calculate net tank volume {#net-tank-volume}
The volume must then be converted from cubic meter to liter. A portion of the volume of the obelisk is lost to the internal structure of the integral tanks (e.g., ribs), with a reduction factor $ f_{\text{volume,usable}} = 0.95$.
Additionally, the expansion of the fuel due to heating must be considered, with a temperature expansion allowance of $ a_{\text{temperature,expansion}} = 0.95$. Thus, the wing tank volume is calculated as:

$$
    V_{\text{tank}} = f_{\text{volume,usable}} \cdot a_{\text{temperature,expansion}} \cdot V_{\text{obelisk}}
$$

!!! note 
    As the wing has a vent tank at each wing tip to allow for the thermodynamic expansion of the fuel, this factor is `1.0` for the wing tanks.

#### Calculate energy {#calculate-energy}
Using the volumetric energy density of kerosene $\eta_{\text{v,kerosene}}$, the energy contained in each tank can be determined:

$$
    V_{\text{tank}} = \eta_{\text{v,kerosene}} \cdot V_{\text{obelisk}}
$$

### Additional center tank
The module allows the installation of an additional center tank in the form of an LD3-45 container. The process includes the following steps:

1. **Height check**: The program first verifies whether the cargo compartment has sufficient height to accommodate the container.
    - If insufficient height is detected: All output values related to the center tank are set to zero.
    - If sufficient height is detected: The installation proceeds.
2. **Installation placement**: The LD3-45 container is positioned 10 cm behind the end of the landing gear bay, aligning approximately with the trailing edge of the wing.
3. **Container data**: The volume and dimensions of the LD3-45 container are predefined and referenced from the Lufthansa Cargo website<sup>[2]</sup>.

The energy contained in an additional center tank is calculated by first determining the usable volume and then taking into account the volumetric energy density of kerosene. With the known factors $ f_{\text{volume,usable}}$ and $a_{\text{temperature,expansion}}$, the usable volume of an additional center tank results in $V_{\text{ACT,usable}} = 3,068.5\text{ L}$ which is equal to an energy amount of $ E_{\text{ACT}} = 103,818.1\text{ MJ}$.

#### Limitations
**Single Center Tank Limit:** The program currently supports the calculation of only one additional center tank. Attempts to add more tanks will not be processed.

!!! note 
    Verify that the cargo compartment dimensions are correctly provided to ensure accurate installation.

### Trim tank {#trim-tank}
The procedure for the trim tank is the same as for the wing tanks. The geometry of the horizontal stabilizer is simplified and divided into several obelisks whose volume is calculated. The usable volume is then determined, taking the already known factors $ f_{\text{volume,usable}}$ and $ a_{\text{temperature,expansion}}$ into account. Finally, the available energy is determined.

<!-- ### Assumptions and tool limitations
tbd. :construction -->
<!-- - All tanks with the same energy carrier (e.g. liquid_hydrogen) have the same design parameters -->
<!-- - Design point for tank design: Point B, design mission -->
<!-- - Der Rumpf wird gestretcht. Es gibt keine Option, die Kabine kleiner zu machen bisher. -->
<!-- - Positionsangaben beziehen sich immer auf vordersten und innersten Punkt, wobei z die Mitte der Ebene angibt (dabei liegt die Annahme zugrunde, dass beim wing auch immer die center line des profils bei der z-Koordinate angegeben wird) -->

## Liquid hydrogen tanks {#liquid-hydrogen-tanks}
tbd. :construction:

---
<sup>[1]</sup> E. Torenbeek, 1982. *Synthesis of Subsonic Airplane Design*.<br>
<sup>[2]</sup> Lufthansa Cargo, 2024. *Our containers*. URL: https://www.lufthansa-cargo.com/de/fleet-ulds/ulds/containers